@@ -13,7 +13,7 @@ import pathlib2
 
 import requests
 import cfscrape
-from webbot.webbot import Browser
+from .browser import StealthBrowser as Browser
 from appdirs import user_data_dir
 
 
@@ -65,11 +65,11 @@ class NoLogginException(Exception):
         LOGGER.warning('Not logged in')
 
 
-class NoPHPsessidException(Exception):
-    """Raise exception when there is no PHPsessid found"""
+class NoCookieException(Exception):
+    """Raise exception when there is no cookie found"""
     def __init__(self, *args, **kwargs):
         Exception.__init__(self, *args, **kwargs)
-        LOGGER.warning('No PHPsessid found')
+        LOGGER.warning('No cookie found')
 
 
 def session_handler(func):
@@ -103,6 +103,7 @@ class AuthenticationHandler:
 
     def __init__(self, show_window=False):
         self.show_window = show_window
+        self.LOGGER = LOGGER
         LOGGER.info('Initialize authentication handler, show window: "%s"',
                     self.show_window)
 
@@ -115,11 +116,12 @@ class AuthenticationHandler:
         self.login()
 
     def login(self):
+        self.remove_cookie(self.username)
         """Login user if needed"""
         LOGGER.info('"%s": start login, method: "%s"',
                     self.username, self.login_method)
-        cookie = self.get_cookie(self.username)
-        if cookie is None:
+        cookies = self.get_cookies(self.username)
+        if not cookies:
             LOGGER.info('"%s": no cookie, new login, method "%s"',
                         self.username, self.login_method)
             if self.login_method not in [
@@ -143,27 +145,61 @@ class AuthenticationHandler:
                 browser = method_dict[self.login_method](browser, auth_text)
             else:
                 LOGGER.info(
-                        '"%s": Invallid loggin method "%s"',
+                        '"%s": Invalid login method "%s"',
                         self.username, self.login_method
                     )
                 sys.exit()
 
-            LOGGER.info('"%s": Get cookie', self.username)
-            phpsessid = browser.get_cookie('PHPSESSID')
-            if phpsessid:
+            LOGGER.info('"%s": Get PHPSESSID', self.username)
+            browser_cookie = browser.get_cookie('PHPSESSID')
+            if browser_cookie:
+                expiry = browser_cookie.get('expiry', None)
+                value = browser_cookie.get('value', None)
+                LOGGER.info('"{}": "value": {}, "expiry": {}'.format(
+                        self.username, value, expiry
+                    ))
                 cookie = self.create_cookie(
-                    phpsessid.get('expiry', None),
-                    phpsessid.get('value', None)
-                )
-                self.write_cookie(self.username, cookie)
+                        'PHPSESSID',
+                        expiry,
+                        value
+                    )
+                cookies.append(cookie)
             else:
-                raise NoPHPsessidException()
+                raise NoCookieException()
+
+            # TODO: what's up with 'rival/googles'
+
+            cookie_names = ['rr_f']
+            for cookie_name in cookie_names:
+                browser_cookie = browser.get_cookie(cookie_name)
+                if browser_cookie:
+                    LOGGER.info('"{}": Get {}'.format(
+                        self.username, cookie_name
+                    ))
+                    expiry = browser_cookie.get('expiry', None)
+                    value = browser_cookie.get('value', None)
+                    cookies.append(
+                        self.create_cookie(
+                            cookie_name,
+                            expiry,
+                            value
+                        )
+                    )
+                    LOGGER.info('"{}": "value": {}, "expiry": {}'.format(
+                            self.username, value, expiry
+                        ))
+                else:
+                    raise NoCookieException()
+
+            self.write_cookies(self.username, cookies)
             LOGGER.debug('"%s": closing login tab', self.username)
             browser.close_current_tab()
+        else:
+            LOGGER.info('Cookies found')
 
         self.session = cfscrape.CloudflareScraper()
-        self.cookie = cookie
-        self.session.cookies.set(**cookie)
+        for cookie in cookies:
+            self.session.cookies.set(**cookie)
 
         LOGGER.debug('"%s": set the var_c', self.username)
         response = self.session.get('https://rivalregions.com/#overview')
@@ -180,22 +216,31 @@ class AuthenticationHandler:
         LOGGER.info('"%s": Login method Google', self.username)
         auth_text1 = auth_text.split('\t<a href="')
         auth_text2 = auth_text1[1].split('" class="sa')
+        time.sleep(1)
         browser.go_to(auth_text2[0])
 
         LOGGER.info('"%s": Typing in username', self.username)
         browser.type(self.username, into='Email')
 
         LOGGER.info('"%s": pressing next button', self.username)
-        browser.click(css_selector="#identifierNext button")
+        browser.click(css_selector="#next")
         time.sleep(2)
 
         LOGGER.info('"%s": Typing in password', self.username)
         browser.type(self.password, css_selector="input")
 
         LOGGER.info('"%s": pressing sign in button', self.username)
-        browser.click(css_selector=".sa_sn.float_left.imp.gogo")
-        time.sleep(2)
+        browser.click(css_selector="#submit")
+        time.sleep(3)
 
+        # Some why it wont click and login immediately. This seems to work
+        time.sleep(1)
+        browser.go_to(auth_text2[0])
+        time.sleep(1)
+        browser.go_to(auth_text2[0])
+        time.sleep(1)
+        browser.click(css_selector="#sa_add2 > div:nth-child(4) > a.sa_link.gogo > div")
+        time.sleep(3)
         return browser
 
     # IDK if this is working
@@ -232,52 +277,61 @@ class AuthenticationHandler:
         return browser
 
     @classmethod
-    def write_cookie(cls, username, cookie):
+    def write_cookies(cls, username, passed_cookies):
         """Write cookie to file"""
-        LOGGER.info('Saving cookie for "%s"', username)
+        LOGGER.info('"%s": Saving cookie', username)
         cookies = None
         try:
             with open('{}/cookies.json'.format(DATA_DIR), 'r') as cookies_file:
                 cookies = json.load(cookies_file)
+            if not cookies:
+                raise FileNotFoundError # raise error as if file hadn't been found
         except FileNotFoundError:
-            cookies = {}
-        cookies[username] = {
-            'expires': cookie['expires'],
-            'value': cookie['value'],
-        }
+            cookies = {username : {}}
+        if username not in cookies:
+            cookies[username] = {}
+        for cookie in passed_cookies:
+            cookies[username][cookie['name']] = {
+                'expiry': cookie['expires'],
+                'value': cookie['value'],
+            }
+
         with open('{}/cookies.json'.format(DATA_DIR), 'w+') as cookies_file:
             json.dump(cookies, cookies_file)
-        LOGGER.info('Saved cookie for "%s"', username)
+        LOGGER.info('"%s": Saved cookie for', username)
 
     @classmethod
-    def get_cookie(cls, username):
+    def get_cookies(cls, username):
         """Read cookies for username"""
         LOGGER.info('"%s": Reading cookie', username)
+        cookies = []
         try:
             with open('{}/cookies.json'.format(DATA_DIR), 'r') as cookies_file:
-                cookies = json.load(cookies_file)
-                for cookie_username, cookie in cookies.items():
+                cookies_data = json.load(cookies_file)
+                for cookie_username, user_cookies in cookies_data.items():
                     if cookie_username == username:
-                        LOGGER.info('"%s": Found cookie', username)
-                        expires = datetime.fromtimestamp(
-                                int(cookie['expires'])
-                            )
-                        if datetime.now() >= expires:
-                            LOGGER.info('"%s": Cookie is expired', username)
-                            return None
-                        cookie = cls.create_cookie(
-                            cookie['expires'],
-                            cookie['value'],
-                        )
-                        return cookie
+                        LOGGER.info('"%s": Found cookies', username)
+                        for cookie_name, cookie in user_cookies.items():
+                            expires = datetime.fromtimestamp(
+                                    int(cookie['expiry'])
+                                )
+                            if datetime.now() >= expires:
+                                LOGGER.info('"%s": Cookie is expired', username)
+                                return None
+                            cookies.append(cls.create_cookie(
+                                cookie_name,
+                                cookie['expiry'],
+                                cookie['value'],
+                            ))
+                        return cookies
         except FileNotFoundError:
             pass
-        return None
+        return cookies
 
     @classmethod
     def remove_cookie(cls, username):
         """Remove cookie from storage"""
-        LOGGER.info('Removing cookie for "%s"', username)
+        LOGGER.info('"%s": Removing cookie for', username)
         cookies = None
         try:
             with open('{}/cookies.json'.format(DATA_DIR), 'r') as cookies_file:
@@ -287,17 +341,17 @@ class AuthenticationHandler:
         cookies.pop(username, None)
         with open('{}/cookies.json'.format(DATA_DIR), 'w+') as cookies_file:
             json.dump(cookies, cookies_file)
-        LOGGER.info('Removed cookie for "%s"', username)
+        LOGGER.info('"%s": Removed cookie', username)
 
     @staticmethod
-    def create_cookie(expires, value):
+    def create_cookie(name, expiry, value):
         """Create cookie"""
         return {
             'domain': 'rivalregions.com',
-            'name': 'PHPSESSID',
+            'name': name,
             'path': '/',
             'secure': False,
-            'expires': expires,
+            'expires': expiry,
             'value': value,
         }
 
@@ -311,7 +365,7 @@ class AuthenticationHandler:
         if add_var_c:
             params['c'] = self.var_c
 
-        LOGGER.debug(
+        LOGGER.info(
                 '"%s" GET: "%s" var_c: %s', self.username, path, add_var_c
             )
         if self.session:
@@ -337,7 +391,7 @@ class AuthenticationHandler:
             data = {}
         data['c'] = self.var_c
 
-        LOGGER.debug('"%s" POST: "%s"', self.username, path)
+        LOGGER.info('"%s" POST: "%s"', self.username, path)
         if self.session:
             response = self.session.post(
                 "https://rivalregions.com/{}".format(path),
@@ -355,14 +409,15 @@ class AuthenticationHandler:
     @session_handler
     def send_chat(self, language, message):
         """send chat message"""
-        LOGGER.debug('"%s" CHAT: language %s', self.username, language)
+        LOGGER.info('"%s" CHAT: language %s', self.username, language)
         if self.session:
             response = self.session.get("https://rivalregions.com/#overview")
             if "Session expired, please, reload the page" in response.text:
                 raise SessionExpireException()
             browser = Browser(showWindow=self.show_window)
             browser.go_to('https://rivalregions.com/')
-            browser.add_cookie(self.get_cookie(self.username))
+            for cookie in self.get_cookies(self.username):
+                browser.add_cookie(cookie)
             browser.go_to(
                     'https://rivalregions.com/#slide/chat/lang_{}'
                     .format(language)
@@ -371,7 +426,10 @@ class AuthenticationHandler:
             time.sleep(2)
             browser.type(message, id='message')
             browser.click(id='chat_send')
-            LOGGER.info('language %s: finished sending message', language)
+            LOGGER.info(
+                '"%s" CHAT: language %s, finished sending message',
+                self.username, language
+            )
             browser.close_current_tab()
         else:
             raise NoLogginException()
@@ -379,14 +437,15 @@ class AuthenticationHandler:
     @session_handler
     def send_personal_message(self, user_id, message):
         """send personal message"""
-        LOGGER.debug('"%s" PM: user id %s', self.username, user_id)
+        LOGGER.info('"%s" PM: user id %s', self.username, user_id)
         if self.session:
             response = self.session.get("https://rivalregions.com/#overview")
             if "Session expired, please, reload the page" in response.text:
                 raise SessionExpireException()
             browser = Browser(showWindow=self.show_window)
             browser.go_to('https://rivalregions.com/')
-            browser.add_cookie(self.get_cookie(self.username))
+            for cookie in self.get_cookies(self.username):
+                browser.add_cookie(cookie)
             browser.go_to(
                     'https://rivalregions.com/#messages/{}'.format(user_id)
                 )
@@ -394,7 +453,9 @@ class AuthenticationHandler:
             time.sleep(2)
             browser.type(message, id='message')
             browser.click(id='chat_send')
-            LOGGER.info('user %s: finished sending message', user_id)
+            LOGGER.info(
+                    '"%s" PM: user id %s, finished sending message',
+                    self.username, user_id)
             browser.close_current_tab()
         else:
             raise NoLogginException()
@@ -402,8 +463,8 @@ class AuthenticationHandler:
     @session_handler
     def send_conference_message(self, conference_id, message):
         """send conference message"""
-        LOGGER.debug(
-                '"%s" CONF: conference id %s',
+        LOGGER.info(
+                '"%s" CONF: id %s',
                 self.username, conference_id
             )
         if self.session:
@@ -412,7 +473,8 @@ class AuthenticationHandler:
                 raise SessionExpireException()
             browser = Browser(showWindow=self.show_window)
             browser.go_to('https://rivalregions.com/')
-            browser.add_cookie(self.get_cookie(self.username))
+            for cookie in self.get_cookies(self.username):
+                browser.add_cookie(cookie)
             browser.go_to(
                     'https://rivalregions.com/#slide/conference/{}'
                     .format(conference_id)
@@ -433,9 +495,9 @@ class AuthenticationHandler:
                                 ' '.join(tmp_sentence)
                             )
                         LOGGER.info(
-                            'conference %s: next message length: %s',
-                            conference_id, len(message)
-                        )
+                                '"%s" CONF: id %s, next message length: %s',
+                                self.username, conference_id, len(message)
+                            )
                         browser.type(message, id='message')
                         browser.click(id='chat_send')
                         sentence_character_count = 0
@@ -478,8 +540,8 @@ class AuthenticationHandler:
     @session_handler
     def send_conference_notification(self, conference_id, message, sound):
         """send conference notification"""
-        LOGGER.debug(
-                '"%s" CONF: conference id %s notification ',
+        LOGGER.info(
+                '"%s" CONF: id %s notification ',
                 self.username, conference_id
             )
         data = {
@@ -489,7 +551,6 @@ class AuthenticationHandler:
         }
 
         if self.session:
-            LOGGER.info('conference %s: sending notification', conference_id)
             response = self.session.post(
                 "https://rivalregions.com/rival/konffcm/{}/".format(
                     conference_id
@@ -503,4 +564,8 @@ class AuthenticationHandler:
                 raise SessionExpireException()
         else:
             raise NoLogginException()
+        LOGGER.info(
+                '"%s" CONF: id %s send notification ',
+                self.username, conference_id
+            )
         return response.text
